@@ -1,11 +1,13 @@
-from typing import Dict, Any, Optional
-import numpy as np
 import collections
+from typing import TYPE_CHECKING, Any, Dict, Optional
+
+import numpy as np
+
+if TYPE_CHECKING:
+    import xarray as xr
 
 from qcodes.utils.helpers import DelegateAttributes, full_class, warn_units
 
-import xarray as xr
-
 
 class DataArray(DelegateAttributes):
 
@@ -533,19 +535,22 @@ class DataArray(DelegateAttributes):
         warn_units('DataArray', self)
         return self.unit
 
-    def to_xarray(self) -> xr.DataArray:
+    def to_xarray(self) -> "xr.DataArray":
         """ Return this DataArray as an xarray dataarray
 
         Returns:
             DataArray in xarray format
         """
+        import xarray as xr
         xarray_dictionary = data_array_to_xarray_dictionary(self)
         xarray_dataarray = xr.DataArray.from_dict(xarray_dictionary)
         return xarray_dataarray
 
     @classmethod
-    def from_xarray(cls, xarray_dataarray: xr.DataArray, array_id: Optional[str] = None) -> 'DataArray':
-        """ Create a DataArray from an xarray DataArray
+    def from_xarray(
+        cls, xarray_dataarray: "xr.DataArray", array_id: Optional[str] = None
+    ) -> "DataArray":
+        """Create a DataArray from an xarray DataArray
 
         Args:
             array_id: Array id for the new DataArray. If None, then use the first data variable from the argument
@@ -584,7 +589,7 @@ def data_array_to_xarray_dictionary(data_array: DataArray) -> Dict[str, Any]:
         data_dictionary["data"] = data
     else:
         if data_array.set_arrays:
-            data_dictionary["dims"] = tuple([a.array_id for a in data_array.set_arrays])
+            data_dictionary["dims"] = tuple(a.array_id for a in data_array.set_arrays)
             data_dictionary["depends_on"] = data_dictionary["dims"]
         data_dictionary["data"] = data_array.ndarray
 
