@@ -4,9 +4,8 @@ to measure and storing results. The user is expected to mainly interact with it
 using the :class:`.Measurement` class.
 """
 
-
+import collections
 import io
-import json
 import logging
 import traceback as tb_module
 import warnings
@@ -16,6 +15,7 @@ from numbers import Number
 from time import perf_counter
 from types import TracebackType
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     Dict,
@@ -36,11 +36,11 @@ import numpy as np
 
 import qcodes as qc
 import qcodes.utils.validators as vals
-from qcodes import Station
-from qcodes.dataset.data_set import (
-    VALUE,
-    DataSet,
-    load_by_guid,
+from qcodes.dataset.data_set import VALUE, DataSet, load_by_guid
+from qcodes.dataset.data_set_in_memory import DataSetInMem
+from qcodes.dataset.data_set_protocol import (
+    DataSetProtocol,
+    DataSetType,
     res_type,
     setpoints_type,
     values_type,
@@ -55,7 +55,6 @@ from qcodes.dataset.descriptions.rundescriber import RunDescriber
 from qcodes.dataset.descriptions.versioning.rundescribertypes import Shapes
 from qcodes.dataset.experiment_container import Experiment
 from qcodes.dataset.export_config import get_data_export_automatic
-from qcodes.dataset.linked_datasets.links import Link
 from qcodes.instrument.delegate.grouped_parameter import GroupedParameter
 from qcodes.instrument.parameter import (
     ArrayParameter,
@@ -65,8 +64,11 @@ from qcodes.instrument.parameter import (
     _BaseParameter,
     expand_setpoints_helper,
 )
+from qcodes.station import Station
 from qcodes.utils.delaykeyboardinterrupt import DelayedKeyboardInterrupt
-from qcodes.utils.helpers import NumpyJSONEncoder
+
+if TYPE_CHECKING:
+    from qcodes.dataset.sqlite.connection import ConnectionPlus
 
 log = logging.getLogger(__name__)
 
@@ -89,30 +91,36 @@ class DataSaver:
 
     default_callback: Optional[Dict[Any, Any]] = None
 
-    def __init__(self, dataset: DataSet,
-                 write_period: float,
-                 interdeps: InterDependencies_) -> None:
+    def __init__(
+        self,
+        dataset: DataSetProtocol,
+        write_period: float,
+        interdeps: InterDependencies_,
+    ) -> None:
         self._dataset = dataset
-        if DataSaver.default_callback is not None \
-                and 'run_tables_subscription_callback' \
-                    in DataSaver.default_callback:
-            callback = DataSaver.default_callback[
-                'run_tables_subscription_callback']
-            min_wait = DataSaver.default_callback[
-                'run_tables_subscription_min_wait']
-            min_count = DataSaver.default_callback[
-                'run_tables_subscription_min_count']
-            snapshot = dataset.get_metadata('snapshot')
-            self._dataset.subscribe(callback,
-                                    min_wait=min_wait,
-                                    min_count=min_count,
-                                    state={},
-                                    callback_kwargs={'run_id':
-                                                     self._dataset.run_id,
-                                                     'snapshot': snapshot})
-        default_subscribers = qc.config.subscription.default_subscribers
-        for subscriber in default_subscribers:
-            self._dataset.subscribe_from_config(subscriber)
+        if (
+            DataSaver.default_callback is not None
+            and "run_tables_subscription_callback" in DataSaver.default_callback
+        ):
+            callback = DataSaver.default_callback["run_tables_subscription_callback"]
+            min_wait = DataSaver.default_callback["run_tables_subscription_min_wait"]
+            min_count = DataSaver.default_callback["run_tables_subscription_min_count"]
+            snapshot = dataset.metadata["snapshot"]
+            if isinstance(self._dataset, DataSet):
+                self._dataset.subscribe(
+                    callback,
+                    min_wait=min_wait,
+                    min_count=min_count,
+                    state={},
+                    callback_kwargs={
+                        "run_id": self._dataset.run_id,
+                        "snapshot": snapshot,
+                    },
+                )
+        if isinstance(self._dataset, DataSet):
+            default_subscribers = qc.config.subscription.default_subscribers
+            for subscriber in default_subscribers:
+                self._dataset.subscribe_from_config(subscriber)
 
         self._interdeps = interdeps
         self.write_period = float(write_period)
@@ -120,7 +128,7 @@ class DataSaver:
         self._results: List[Dict[str, VALUE]] = []
         self._last_save_time = perf_counter()
         self._known_dependencies: Dict[str, List[str]] = {}
-        self.parent_datasets: List[DataSet] = []
+        self.parent_datasets: List[DataSetProtocol] = []
 
         for link in self._dataset.parent_dataset_links:
             self.parent_datasets.append(load_by_guid(link.tail))
@@ -152,6 +160,7 @@ class DataSaver:
             ValueError: If the shapes of parameters do not match, i.e. if a
                 parameter gets values of a different shape than its setpoints
                 (the exception being that setpoints can always be scalar)
+            ValueError: If multiple results are given for the same parameter.
             ParameterTypeError: If a parameter is given a value not matching
                 its type.
         """
@@ -166,6 +175,16 @@ class DataSaver:
         parameter_names = tuple(partial_result[0].full_name
                                 if isinstance(partial_result[0], _BaseParameter) else partial_result[0]
                                 for partial_result in res_tuple)
+        if len(set(parameter_names)) != len(parameter_names):
+            non_unique = [
+                item
+                for item, count in collections.Counter(parameter_names).items()
+                if count > 1
+            ]
+            raise ValueError(
+                f"Not all parameter names are unique. "
+                f"Got multiple values for {non_unique}"
+            )
 
         for partial_result in res_tuple:
             parameter = partial_result[0]
@@ -400,10 +419,10 @@ class DataSaver:
         toplevel_params = (set(self._interdeps.dependencies)
                            .intersection(set(results_dict)))
         for toplevel_param in toplevel_params:
-            required_shape = np.shape(results_dict[toplevel_param])
+            required_shape = np.shape(np.array(results_dict[toplevel_param]))
             for setpoint in self._interdeps.dependencies[toplevel_param]:
                 # a setpoint is allowed to be a scalar; shape is then ()
-                setpoint_shape = np.shape(results_dict[setpoint])
+                setpoint_shape = np.shape(np.array(results_dict[setpoint]))
                 if setpoint_shape not in [(), required_shape]:
                     raise ValueError(f'Incompatible shapes. Parameter '
                                      f"{toplevel_param.name} has shape "
@@ -454,7 +473,7 @@ class DataSaver:
         return self._dataset.number_of_results
 
     @property
-    def dataset(self) -> DataSet:
+    def dataset(self) -> DataSetProtocol:
         return self._dataset
 
 
@@ -485,8 +504,10 @@ class Runner:
         write_in_background: bool = False,
         shapes: Optional[Shapes] = None,
         in_memory_cache: bool = True,
+        dataset_class: DataSetType = DataSetType.DataSet,
     ) -> None:
 
+        self._dataset_class = dataset_class
         self.write_period = self._calculate_write_period(write_in_background,
                                                          write_period)
 
@@ -506,6 +527,7 @@ class Runner:
         self._extra_log_info = extra_log_info
         self._write_in_background = write_in_background
         self._in_memory_cache = in_memory_cache
+        self.ds: DataSetProtocol
 
     @staticmethod
     def _calculate_write_period(
@@ -532,56 +554,79 @@ class Runner:
         for func, args in self.enteractions:
             func(*args)
 
+        dataset_class: Type[DataSetProtocol]
+
         # next set up the "datasaver"
         if self.experiment is not None:
-            self.ds = qc.new_data_set(
-                self.name, self.experiment.exp_id,
-                conn=self.experiment.conn,
-                in_memory_cache=self._in_memory_cache
+            exp_id: Optional[int] = self.experiment.exp_id
+            path_to_db: Optional[str] = self.experiment.path_to_db
+            conn: Optional["ConnectionPlus"] = self.experiment.conn
+        else:
+            exp_id = None
+            path_to_db = None
+            conn = None
+
+        if self._dataset_class is DataSetType.DataSet:
+            self.ds = DataSet(
+                name=self.name,
+                exp_id=exp_id,
+                conn=conn,
+                in_memory_cache=self._in_memory_cache,
+            )
+        elif self._dataset_class is DataSetType.DataSetInMem:
+            if self._in_memory_cache is False:
+                raise RuntimeError(
+                    "Cannot disable the in memory cache for a "
+                    "dataset that is only in memory."
+                )
+            self.ds = DataSetInMem._create_new_run(
+                name=self.name,
+                exp_id=exp_id,
+                path_to_db=path_to_db,
             )
         else:
-            self.ds = qc.new_data_set(
-                self.name,
-                in_memory_cache=self._in_memory_cache
-            )
+            raise RuntimeError("Does not support any other dataset classes")
 
         # .. and give the dataset a snapshot as metadata
         if self.station is None:
-            station = qc.Station.default
+            station = Station.default
         else:
             station = self.station
 
-        if station:
-            self.ds.add_snapshot(json.dumps({'station': station.snapshot()},
-                                            cls=NumpyJSONEncoder))
-
-        if self._interdependencies == InterDependencies_():
-            raise RuntimeError("No parameters supplied")
+        if station is not None:
+            snapshot = station.snapshot()
         else:
-            self.ds.set_interdependencies(self._interdependencies,
-                                          self._shapes)
+            snapshot = {}
 
-        links = [Link(head=self.ds.guid, **pdict)
-                 for pdict in self._parent_datasets]
-        self.ds.parent_dataset_links = links
-        self.ds.mark_started(start_bg_writer=self._write_in_background)
+        self.ds.prepare(
+            snapshot=snapshot,
+            interdeps=self._interdependencies,
+            write_in_background=self._write_in_background,
+            shapes=self._shapes,
+            parent_datasets=self._parent_datasets,
+        )
 
         # register all subscribers
-        for (callble, state) in self.subscribers:
-            # We register with minimal waiting time.
-            # That should make all subscribers be called when data is flushed
-            # to the database
-            log.debug(f'Subscribing callable {callble} with state {state}')
-            self.ds.subscribe(callble, min_wait=0, min_count=1, state=state)
+        if isinstance(self.ds, DataSet):
+            for (callble, state) in self.subscribers:
+                # We register with minimal waiting time.
+                # That should make all subscribers be called when data is flushed
+                # to the database
+                log.debug(f"Subscribing callable {callble} with state {state}")
+                self.ds.subscribe(callble, min_wait=0, min_count=1, state=state)
 
-        print(f'Starting experimental run with id: {self.ds.run_id}.'
-              f' {self._extra_log_info}')
-        log.info(f'Starting measurement with guid: {self.ds.guid}, '
-                 f'sample_name: "{self.ds.sample_name}", '
-                 f'exp_name: "{self.ds.exp_name}", '
-                 f'ds_name: "{self.ds.name}". '
-                 f'{self._extra_log_info}')
-        log.info(f'Using background writing: {self._write_in_background}')
+        print(
+            f"Starting experimental run with id: {self.ds.captured_run_id}."
+            f" {self._extra_log_info}"
+        )
+        log.info(
+            f"Starting measurement with guid: {self.ds.guid}, "
+            f'sample_name: "{self.ds.sample_name}", '
+            f'exp_name: "{self.ds.exp_name}", '
+            f'ds_name: "{self.ds.name}". '
+            f"{self._extra_log_info}"
+        )
+        log.info(f"Using background writing: {self._write_in_background}")
 
         self.datasaver = DataSaver(
                             dataset=self.ds,
@@ -624,7 +669,8 @@ class Runner:
                 self.datasaver.export_data()
             log.info(f'Finished measurement with guid: {self.ds.guid}. '
                      f'{self._extra_log_info}')
-            self.ds.unsubscribe_all()
+            if isinstance(self.ds, DataSet):
+                self.ds.unsubscribe_all()
 
 
 T = TypeVar('T', bound='Measurement')
@@ -646,9 +692,12 @@ class Measurement:
             'results' is used for the dataset.
     """
 
-    def __init__(self, exp: Optional[Experiment] = None,
-                 station: Optional[qc.Station] = None,
-                 name: str = '') -> None:
+    def __init__(
+        self,
+        exp: Optional[Experiment] = None,
+        station: Optional[Station] = None,
+        name: str = "",
+    ) -> None:
         self.exitactions: List[ActionType] = []
         self.enteractions: List[ActionType] = []
         self.subscribers: List[SubscriberType] = []
@@ -724,8 +773,8 @@ class Measurement:
         return tuple(depends_on), tuple(inf_from)
 
     def register_parent(
-            self: T, parent: DataSet, link_type: str,
-            description: str = "") -> T:
+        self: T, parent: DataSetProtocol, link_type: str, description: str = ""
+    ) -> T:
         """
         Register a parent for the outcome of this measurement
 
@@ -1154,8 +1203,12 @@ class Measurement:
                                              shapes=shapes)
         self._shapes = shapes
 
-    def run(self, write_in_background: Optional[bool] = None,
-            in_memory_cache: bool = True) -> Runner:
+    def run(
+        self,
+        write_in_background: Optional[bool] = None,
+        in_memory_cache: bool = True,
+        dataset_class: DataSetType = DataSetType.DataSet,
+    ) -> Runner:
         """
         Returns the context manager for the experimental run
 
@@ -1168,17 +1221,24 @@ class Measurement:
                 read from the ``qcodesrc.json`` config file.
             in_memory_cache: Should measured data be keep in memory
                 and available as part of the `dataset.cache` object.
+            dataset_class: Enum representing the Class used to store data
+                with.
         """
         if write_in_background is None:
             write_in_background = qc.config.dataset.write_in_background
-        return Runner(self.enteractions, self.exitactions,
-                      self.experiment, station=self.station,
-                      write_period=self._write_period,
-                      interdeps=self._interdeps,
-                      name=self.name,
-                      subscribers=self.subscribers,
-                      parent_datasets=self._parent_datasets,
-                      extra_log_info=self._extra_log_info,
-                      write_in_background=write_in_background,
-                      shapes=self._shapes,
-                      in_memory_cache=in_memory_cache)
+        return Runner(
+            self.enteractions,
+            self.exitactions,
+            self.experiment,
+            station=self.station,
+            write_period=self._write_period,
+            interdeps=self._interdeps,
+            name=self.name,
+            subscribers=self.subscribers,
+            parent_datasets=self._parent_datasets,
+            extra_log_info=self._extra_log_info,
+            write_in_background=write_in_background,
+            shapes=self._shapes,
+            in_memory_cache=in_memory_cache,
+            dataset_class=dataset_class,
+        )
